@@ -1,28 +1,12 @@
 import os
-from contextlib import suppress
 
 import pytest
 
-try:
-    import psutil
-except ImportError:
-    psutil = None
-
 try:
     import hypothesis
 except ImportError:
     hypothesis = None
 
-try:
-    from os import process_cpu_count
-except ImportError:
-    process_cpu_count = None
-
-try:
-    from os import sched_getaffinity
-except ImportError:
-    sched_getaffinity = None
-
 
 def test_default_threads(pytester):
     """Make sure that pytest accepts our fixture."""
@@ -301,9 +285,9 @@ def test_num_parallel_threads_fixture(pytester):
             "*::test_should_yield_global_threads PARALLEL PASSED*",
             "*::test_should_yield_marker_threads PARALLEL PASSED*",
             "*::test_single_threaded PASSED*",
-            "*1 tests were not run in parallel because of use of "
+            "*1 test was not run in parallel because of use of "
             "thread-unsafe functionality, to list the tests that "
-            "were skipped, re-run while setting PYTEST_RUN_PARALLEL_VERBOSE=1"
+            "were not run in parallel, re-run while setting PYTEST_RUN_PARALLEL_VERBOSE=1"
             " in your shell environment",
         ]
     )
@@ -321,62 +305,6 @@ def test_num_parallel_threads_fixture(pytester):
     )
 
 
-def test_thread_comp_fixture(pytester):
-    """Test that ThreadComparator works as expected."""
-
-    # create a temporary pytest test module
-    pytester.makepyfile("""
-        import threading
-        import pytest
-
-        class Counter:
-            def __init__(self):
-                self._value = 0
-                self._lock = threading.Lock()
-
-            def get_value_and_increment(self):
-                with self._lock:
-                    value = int(self._value)
-                    self._value += 1
-                return value
-
-        def test_value_comparison(num_parallel_threads, thread_comp):
-            assert num_parallel_threads == 10
-            a = 1
-            b = [2, 'string', 1.0]
-            c = {'a': -4, 'b': 'str'}
-            d = float('nan')
-            e = float('inf')
-            f = {'a', 'b', '#'}
-            thread_comp(a=a, b=b, c=c, d=d, e=e, f=f)
-
-            # Ensure that the comparator can be used again
-            thread_comp(g=4)
-
-        @pytest.fixture
-        def counter(num_parallel_threads):
-            return Counter()
-
-        def test_comparison_fail(thread_comp, counter):
-            a = 4
-            pos = counter.get_value_and_increment()
-            if pos % 2 == 0:
-                a = -1
-            thread_comp(a=a)
-    """)
-
-    # run pytest with the following cmd args
-    result = pytester.runpytest("--parallel-threads=10", "-v")
-
-    # fnmatch_lines does an assertion internally
-    result.stdout.fnmatch_lines(
-        [
-            "*::test_value_comparison PARALLEL PASSED*",
-            "*::test_comparison_fail PARALLEL FAILED*",
-        ]
-    )
-
-
 def test_iterations_marker_one_thread(pytester):
     # create a temporary pytest test module
     pytester.makepyfile("""
@@ -564,294 +492,6 @@ def test_skipif_marker_works(pytester):
     )
 
 
-def test_thread_unsafe_marker(pytester):
-    # create a temporary pytest test module
-    pytester.makepyfile("""
-        import pytest
-
-        @pytest.mark.thread_unsafe
-        def test_should_run_single(num_parallel_threads):
-            assert num_parallel_threads == 1
-
-        @pytest.mark.thread_unsafe(reason='this is thread-unsafe')
-        def test_should_run_single_2(num_parallel_threads):
-            assert num_parallel_threads == 1
-    """)
-
-    # run pytest with the following cmd args
-    result = pytester.runpytest("--parallel-threads=10", "-v")
-
-    # fnmatch_lines does an assertion internally
-    result.stdout.fnmatch_lines(
-        [
-            "*::test_should_run_single PASSED*",
-            "*::test_should_run_single_2 PASSED *thread-unsafe*: this is thread-unsafe*",
-        ]
-    )
-
-    # check that skipping works too
-    result = pytester.runpytest(
-        "--parallel-threads=10", "--skip-thread-unsafe=True", "-v"
-    )
-
-    result.stdout.fnmatch_lines(
-        ["*::test_should_run_single SKIPPED*", "*::test_should_run_single_2 SKIPPED*"]
-    )
-
-
-def test_pytest_warns_detection(pytester):
-    # create a temporary pytest test module
-    pytester.makepyfile("""
-        import pytest
-        import warnings
-        import pytest as pyt
-        import warnings as w
-        from pytest import warns, deprecated_call
-        from warnings import catch_warnings
-
-        warns_alias = warns
-
-        def test_single_thread_warns_1(num_parallel_threads):
-            with pytest.warns(UserWarning):
-                warnings.warn('example', UserWarning)
-            assert num_parallel_threads == 1
-
-        def test_single_thread_warns_2(num_parallel_threads):
-            with warns(UserWarning):
-                warnings.warn('example', UserWarning)
-            assert num_parallel_threads == 1
-
-        def test_single_thread_warns_3(num_parallel_threads):
-            with pyt.warns(UserWarning):
-                warnings.warn('example', UserWarning)
-            assert num_parallel_threads == 1
-
-        def test_single_thread_warns_4(num_parallel_threads):
-            with warns_alias(UserWarning):
-                warnings.warn('example', UserWarning)
-            assert num_parallel_threads == 1
-    """)
-
-    # run pytest with the following cmd args
-    result = pytester.runpytest("--parallel-threads=10", "-v")
-
-    # fnmatch_lines does an assertion internally
-    result.stdout.fnmatch_lines(
-        [
-            "*::test_single_thread_warns_1 PASSED*",
-            "*::test_single_thread_warns_2 PASSED*",
-            "*::test_single_thread_warns_3 PASSED*",
-            "*::test_single_thread_warns_4 PASSED*",
-        ]
-    )
-
-    # check that skipping works too
-    result = pytester.runpytest(
-        "--parallel-threads=10", "--skip-thread-unsafe=True", "-v"
-    )
-
-    result.stdout.fnmatch_lines(
-        [
-            "*::test_single_thread_warns_1 SKIPPED*",
-            "*::test_single_thread_warns_2 SKIPPED*",
-            "*::test_single_thread_warns_3 SKIPPED*",
-            "*::test_single_thread_warns_4 SKIPPED*",
-        ]
-    )
-
-
-@pytest.mark.skipif(psutil is None, reason="psutil needs to be installed")
-def test_auto_detect_cpus_psutil_affinity(
-    pytester: pytest.Pytester, monkeypatch: pytest.MonkeyPatch
-) -> None:
-    import psutil
-
-    monkeypatch.setattr(
-        psutil.Process, "cpu_affinity", lambda self: list(range(10)), raising=False
-    )
-
-    pytester.makepyfile("""
-        def test_auto_detect_cpus(num_parallel_threads):
-            assert num_parallel_threads == 10
-    """)
-
-    # run pytest with the following cmd args
-    result = pytester.runpytest("--parallel-threads=auto", "-v")
-
-    # fnmatch_lines does an assertion internally
-    result.stdout.fnmatch_lines(
-        [
-            "*::test_auto_detect_cpus PARALLEL PASSED*",
-        ]
-    )
-
-
-@pytest.mark.skipif(psutil is None, reason="psutil needs to be installed")
-def test_auto_detect_cpus_psutil_cpu_count(
-    pytester: pytest.Pytester, monkeypatch: pytest.MonkeyPatch
-) -> None:
-    import psutil
-
-    monkeypatch.delattr(psutil.Process, "cpu_affinity", raising=False)
-    monkeypatch.setattr(psutil, "cpu_count", lambda: 10)
-
-    pytester.makepyfile("""
-        def test_auto_detect_cpus(num_parallel_threads):
-            assert num_parallel_threads == 10
-    """)
-
-    # run pytest with the following cmd args
-    result = pytester.runpytest("--parallel-threads=auto", "-v")
-
-    # fnmatch_lines does an assertion internally
-    result.stdout.fnmatch_lines(
-        [
-            "*::test_auto_detect_cpus PARALLEL PASSED*",
-        ]
-    )
-
-
-@pytest.mark.skipif(
-    process_cpu_count is None, reason="process_cpu_count is available in >=3.13"
-)
-def test_auto_detect_process_cpu_count(
-    pytester: pytest.Pytester, monkeypatch: pytest.MonkeyPatch
-) -> None:
-    with suppress(ImportError):
-        import psutil
-
-        monkeypatch.delattr(psutil.Process, "cpu_affinity", raising=False)
-        monkeypatch.setattr(psutil, "cpu_count", lambda: None)
-
-    monkeypatch.setattr("os.process_cpu_count", lambda: 10)
-
-    pytester.makepyfile("""
-        def test_auto_detect_cpus(num_parallel_threads):
-            assert num_parallel_threads == 10
-    """)
-
-    # run pytest with the following cmd args
-    result = pytester.runpytest("--parallel-threads=auto", "-v")
-
-    # fnmatch_lines does an assertion internally
-    result.stdout.fnmatch_lines(
-        [
-            "*::test_auto_detect_cpus PARALLEL PASSED*",
-        ]
-    )
-
-
-@pytest.mark.skipif(
-    sched_getaffinity is None,
-    reason="sched_getaffinity is available certain platforms only",
-)
-def test_auto_detect_sched_getaffinity(
-    pytester: pytest.Pytester, monkeypatch: pytest.MonkeyPatch
-) -> None:
-    with suppress(ImportError):
-        import psutil
-
-        monkeypatch.delattr(psutil.Process, "cpu_affinity", raising=False)
-        monkeypatch.setattr(psutil, "cpu_count", lambda: None)
-
-    monkeypatch.setattr("os.process_cpu_count", lambda: None, raising=False)
-    monkeypatch.setattr("os.sched_getaffinity", lambda pid: list(range(10)))
-
-    pytester.makepyfile("""
-        def test_auto_detect_cpus(num_parallel_threads):
-            assert num_parallel_threads == 10
-    """)
-
-    # run pytest with the following cmd args
-    result = pytester.runpytest("--parallel-threads=auto", "-v")
-
-    # fnmatch_lines does an assertion internally
-    result.stdout.fnmatch_lines(
-        [
-            "*::test_auto_detect_cpus PARALLEL PASSED*",
-        ]
-    )
-
-
-def test_auto_detect_cpu_count(
-    pytester: pytest.Pytester, monkeypatch: pytest.MonkeyPatch
-) -> None:
-    with suppress(ImportError):
-        import psutil
-
-        monkeypatch.delattr(psutil.Process, "cpu_affinity", raising=False)
-        monkeypatch.setattr(psutil, "cpu_count", lambda: None)
-
-    monkeypatch.setattr("os.process_cpu_count", lambda: None, raising=False)
-    monkeypatch.setattr("os.sched_getaffinity", lambda pid: None, raising=False)
-    monkeypatch.setattr("os.cpu_count", lambda: 10)
-
-    pytester.makepyfile("""
-        def test_auto_detect_cpus(num_parallel_threads):
-            assert num_parallel_threads == 10
-    """)
-
-    # run pytest with the following cmd args
-    result = pytester.runpytest("--parallel-threads=auto", "-v")
-
-    # fnmatch_lines does an assertion internally
-    result.stdout.fnmatch_lines(
-        [
-            "*::test_auto_detect_cpus PARALLEL PASSED*",
-        ]
-    )
-
-
-def test_thread_unsafe_fixtures(pytester):
-    # create a temporary pytest test module
-    pytester.makepyfile("""
-        import pytest
-
-        @pytest.fixture
-        def my_unsafe_fixture():
-            pass
-
-        @pytest.fixture
-        def my_unsafe_fixture_2():
-            pass
-
-        def test_capsys(capsys, num_parallel_threads):
-            assert num_parallel_threads == 1
-
-        def test_monkeypatch(monkeypatch, num_parallel_threads):
-            assert num_parallel_threads == 1
-
-        def test_recwarn(recwarn, num_parallel_threads):
-            assert num_parallel_threads == 1
-
-        def test_custom_fixture_skip(my_unsafe_fixture, num_parallel_threads):
-            assert num_parallel_threads == 1
-
-        def test_custom_fixture_skip_2(my_unsafe_fixture_2, num_parallel_threads):
-            assert num_parallel_threads == 1
-    """)
-
-    pytester.makeini("""
-    [pytest]
-    thread_unsafe_fixtures =
-        my_unsafe_fixture
-        my_unsafe_fixture_2
-    """)
-
-    # run pytest with the following cmd args
-    result = pytester.runpytest("--parallel-threads=10", "-v")
-
-    # fnmatch_lines does an assertion internally
-    result.stdout.fnmatch_lines(
-        [
-            "*::test_capsys PASSED *thread-unsafe*: uses thread-unsafe fixture*",
-            "*::test_recwarn PASSED *thread-unsafe*: uses thread-unsafe fixture*",
-            "*::test_custom_fixture_skip PASSED *thread-unsafe*: uses thread-unsafe fixture*",
-            "*::test_custom_fixture_skip_2 PASSED *thread-unsafe*: uses thread-unsafe fixture*",
-        ]
-    )
-
-
 def test_incompatible_test_item(pytester):
     pytester.makeconftest("""
     import inspect
@@ -879,7 +519,7 @@ def test_incompatible_test_item(pytester):
     def test_incompatible_item():
         assert True
     """)
-    result = pytester.runpytest("--parallel-threads=10", "-v")
+    result = pytester.runpytest("--parallel-threads=10", "-v", "-W", "default")
     result.stdout.fnmatch_lines(
         [
             "*::test_incompatible_item PASSED*",
@@ -929,140 +569,6 @@ def test_known_incompatible_test_item_doesnt_warn(pytester):
     assert "warnings" not in result.parseoutcomes().keys()
 
 
-def test_thread_unsafe_function_attr(pytester):
-    pytester.makepyfile(
-        mod_1="""
-        def to_skip():
-            __thread_safe__ = False
-
-        def not_to_skip():
-            __thread_safe__ = True
-    """
-    )
-
-    pytester.makepyfile(
-        mod_2="""
-        import mod_1
-        from mod_1 import not_to_skip
-
-        def some_fn_calls_skip():
-            mod_1.to_skip()
-
-        def some_fn_should_not_skip():
-            not_to_skip()
-
-        def marked_for_skip():
-            pass
-    """
-    )
-
-    pytester.makepyfile("""
-        import mod_2
-        from mod_2 import some_fn_calls_skip
-
-        def test_should_be_marked_1(num_parallel_threads):
-            mod_2.some_fn_calls_skip()
-            assert num_parallel_threads == 1
-
-        def test_should_not_be_marked(num_parallel_threads):
-            mod_2.some_fn_should_not_skip()
-            assert num_parallel_threads == 10
-
-        def test_should_be_marked_2(num_parallel_threads):
-            mod_2.marked_for_skip()
-            assert num_parallel_threads == 1
-
-        def test_should_be_marked_3(num_parallel_threads):
-            some_fn_calls_skip()
-            assert num_parallel_threads == 1
-    """)
-
-    pytester.makeini("""
-    [pytest]
-    thread_unsafe_functions =
-        mod_2.marked_for_skip
-    """)
-
-    # run pytest with the following cmd args
-    orig = os.environ.get("PYTEST_RUN_PARALLEL_VERBOSE", "0")
-    os.environ["PYTEST_RUN_PARALLEL_VERBOSE"] = "0"
-    result = pytester.runpytest("--parallel-threads=10", "-v")
-
-    # fnmatch_lines does an assertion internally
-    result.stdout.fnmatch_lines(
-        [
-            "*Collected 1 items to run in parallel*",
-            "*::test_should_be_marked_1 PASSED *thread-unsafe*inferred via func.__thread_safe__*",
-            "*::test_should_not_be_marked PARALLEL PASSED*",
-            "*::test_should_be_marked_2 PASSED *thread-unsafe*marked_for_skip*",
-            "*::test_should_be_marked_3 PASSED *thread-unsafe*inferred via func.__thread_safe__*",
-        ]
-    )
-
-    result.stdout.fnmatch_lines(
-        [
-            "*3 tests were not run in parallel because of use of thread-unsafe "
-            "functionality, to list the tests that were skipped, "
-            "re-run while setting PYTEST_RUN_PARALLEL_VERBOSE=1 in your "
-            "shell environment*",
-        ]
-    )
-
-    # re-run with PYTEST_RUN_PARALLEL_VERBOSE=1
-    os.environ["PYTEST_RUN_PARALLEL_VERBOSE"] = "1"
-    result = pytester.runpytest("--parallel-threads=10", "-v")
-    os.environ["PYTEST_RUN_PARALLEL_VERBOSE"] = orig
-
-    result.stdout.fnmatch_lines(
-        [
-            "*Collected 1 items to run in parallel*",
-            "*::test_should_be_marked_1 PASSED *thread-unsafe*: calls thread-unsafe function*",
-            "*::test_should_not_be_marked PARALLEL PASSED*",
-            "*::test_should_be_marked_2 PASSED*",
-            "*::test_should_be_marked_3 PASSED*",
-            "*::test_should_be_marked_1*",
-            "*::test_should_be_marked_2*",
-            "*::test_should_be_marked_3*",
-        ]
-    )
-
-
-@pytest.mark.skipif(hypothesis is None, reason="hypothesis needs to be installed")
-def test_detect_hypothesis(pytester):
-    pytester.makepyfile("""
-    from hypothesis import given, strategies as st, settings, HealthCheck
-
-    @given(a=st.none())
-    @settings(suppress_health_check=[HealthCheck.function_scoped_fixture])
-    def test_uses_hypothesis(a, num_parallel_threads):
-        assert num_parallel_threads == 1
-    """)
-    result = pytester.runpytest("--parallel-threads=10", "-v")
-    result.stdout.fnmatch_lines(
-        [
-            "*::test_uses_hypothesis PASSED*",
-        ]
-    )
-
-
-def test_detect_unittest_mock(pytester):
-    pytester.makepyfile("""
-    import sys
-    from unittest import mock
-
-    @mock.patch("sys.platform", "VAX")
-    def test_uses_mock(num_parallel_threads):
-        assert sys.platform == "VAX"
-        assert num_parallel_threads == 1
-    """)
-    result = pytester.runpytest("--parallel-threads=10", "-v")
-    result.stdout.fnmatch_lines(
-        [
-            r"*::test_uses_mock PASSED*" r"calls thread-unsafe function: mock.patch*",
-        ]
-    )
-
-
 def test_all_tests_in_parallel(pytester):
     pytester.makepyfile("""
     def test_parallel_1(num_parallel_threads):
@@ -1092,39 +598,132 @@ def test_all_tests_in_parallel(pytester):
     )
 
 
-def test_recurse_assign(pytester):
+def test_doctests_marked_thread_unsafe(pytester):
     pytester.makepyfile("""
-    import pytest
-
-    def test_function_recurse_on_assign(num_parallel_threads):
-        w = pytest.warns(UserWarning)
-        assert num_parallel_threads == 1
+    def test_parallel(num_parallel_threads):
+        assert num_parallel_threads == 10
     """)
 
+    pytester.makefile(
+        ".txt",
+        """
+    hello this is a doctest
+    >>> x = 3
+    >>> x
+    3
+    >>> num_parallel_threads = getfixture("num_parallel_threads")
+    >>> num_parallel_threads
+    1
+    """,
+    )
+
     result = pytester.runpytest("--parallel-threads=10", "-v")
     result.stdout.fnmatch_lines(
         [
-            "*::test_function_recurse_on_assign PASSED*",
+            "*::test_parallel PARALLEL PASSED*",
+            "*::test_doctests_marked_thread_unsafe.txt PASSED*",
         ]
     )
 
 
-def test_failed_thread_unsafe(pytester):
+@pytest.mark.skipif(hypothesis is None, reason="hypothesis needs to be installed")
+def test_runs_hypothesis_in_parallel(pytester):
     pytester.makepyfile("""
-    import pytest
+    from hypothesis import given, strategies as st, settings, HealthCheck
 
-    @pytest.mark.thread_unsafe
-    def test1():
-        assert False
+    @given(a=st.none())
+    @settings(suppress_health_check=[HealthCheck.function_scoped_fixture])
+    def test_uses_hypothesis(a, num_parallel_threads):
+        assert num_parallel_threads == 10
     """)
-
     result = pytester.runpytest("--parallel-threads=10", "-v")
+    result.stdout.fnmatch_lines(
+        [
+            "*::test_uses_hypothesis PARALLEL PASSED*",
+        ]
+    )
+
+
+def test_fail_warning_gil_enabled_during_execution(pytester):
+    test_name = "test_fail_warning_gil_enabled_during_execution"
+    pytester.makepyfile(f"""
+    import warnings
+
+    def {test_name}():
+        warnings.warn(
+            "The global interpreter lock (GIL) has been enabled to load module 'module'",
+            RuntimeWarning
+        )
+    """)
+    result = pytester.runpytest("-v", "-W", "default")
     assert result.ret == 1
-    print(result.stdout)
     result.stdout.fnmatch_lines(
         [
-            "*::test1 FAILED *thread-unsafe*: uses thread_unsafe marker*",
-            "* FAILURES *",
-            "*1 failed*",
+            f"*GIL was dynamically re-enabled during test execution of '{test_name}.py::{test_name}' to load module 'module'*"
+        ]
+    )
+
+
+def test_fail_warning_gil_enabled_during_collection(pytester):
+    test_name = "test_fail_warning_gil_enabled_during_collection"
+    pytester.makepyfile(f"""
+    import warnings
+    warnings.warn(
+        "The global interpreter lock (GIL) has been enabled to load module 'module'",
+        RuntimeWarning
+    )
+
+    def {test_name}():
+        assert True
+    """)
+    result = pytester.runpytest("-v", "-W", "default")
+    assert result.ret == 1
+    result.stdout.fnmatch_lines(
+        [
+            "*GIL was dynamically re-enabled during test collection to load module 'module'*"
+        ]
+    )
+
+
+def test_warning_gil_enabled_ignore_option(pytester):
+    pytester.makepyfile("""
+    import warnings
+    warnings.warn(
+        "The global interpreter lock (GIL) has been enabled to load module 'module'",
+        RuntimeWarning
+    )
+
+    def test_warning_gil_enabled_ignore_option():
+        assert True
+    """)
+    result = pytester.runpytest("-v", "--ignore-gil-enabled", "-W", "default")
+    assert result.ret == 0
+
+
+def test_runs_with_xdist(pytester):
+    pytester.makepyfile("""
+    def test_parallel1(num_parallel_threads):
+        assert num_parallel_threads == 10
+
+    def test_parallel2(num_parallel_threads):
+        assert num_parallel_threads == 10
+    """)
+    result = pytester.runpytest("--parallel-threads=10", "-n", "2", "-v")
+    result.stdout.fnmatch_lines(["*test_parallel1*"])
+    result.stdout.fnmatch_lines(["*test_parallel2*"])
+    result.stdout.fnmatch_lines(["*All tests were run in parallel!*"])
+    assert result.ret == 0
+
+
+def test_forever_with_xdist_errors(pytester):
+    pytester.makepyfile("""
+    def test_example():
+        assert True
+    """)
+    result = pytester.runpytest("--forever", "-n", "2")
+    assert result.ret != 0
+    result.stderr.fnmatch_lines(
+        [
+            "*--forever from pytest-run-parallel is incompatible with `-n 2` from pytest-xdist*"
         ]
     )
